@@ -657,8 +657,8 @@ class TestAgent {
       return SECFailure;
     }
 
-    SECITEM_AllocItem(NULL, output, input->len + 2);
-    if (output == NULL || output->data == NULL) {
+    if (output == NULL || output->data == NULL ||
+        output->len != input->len + 2) {
       return SECFailure;
     }
 
@@ -718,9 +718,8 @@ class TestAgent {
       return SECFailure;
     }
 
-    SECITEM_AllocItem(NULL, output, input->len - 4);
-
-    if (output == NULL || output->data == NULL) {
+    if (output == NULL || output->data == NULL ||
+        output->len != input->len - 4) {
       return SECFailure;
     }
 
@@ -785,9 +784,9 @@ class TestAgent {
       std::cerr << "Certificate is too short. " << std::endl;
       return SECFailure;
     }
-    SECITEM_AllocItem(NULL, output, input->len - 1);
 
-    if (output == NULL || output->data == NULL) {
+    if (output == NULL || output->data == NULL ||
+        output->len != input->len - 1) {
       return SECFailure;
     }
 
